__title__ = 'django-activeurl'
__package_name__ = 'django_activeurl'
__version__ = '0.1.10'
__description__ = 'Easy-to-use active URL highlighting for Django'
__email__ = 'hellysmile@gmail.com'
__author__ = 'hellysmile'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright (c) hellysmile@gmail.com'
